@@ -1,4 +1,4 @@
-# -*- coding: utf-8 -*-
+# coding: utf-8
 from .imagemodel import _ImageModel, ImageModel
 
 import torch.nn as nn
@@ -23,6 +23,6 @@ class _Net(_ImageModel):
 
 class Net(ImageModel):
 
-    def __init__(self, name='net', model_class=_Net, **kwargs):
+    def __init__(self, name: str = 'net', model_class: type[_Net] = _Net, **kwargs):
         super().__init__(name=name, model_class=model_class,
                          conv_dim=9216, fc_depth=2, fc_dim=128, **kwargs)
