@@ -1,12 +1,13 @@
-# -*- coding: utf-8 -*-
+# coding: utf-8
 
 
 from trojanzoo.configs import config, Config
 from trojanzoo.datasets import Dataset
 from trojanzoo.environ import env
 from trojanzoo.utils import add_noise, empty_cache, repeat_to_batch, to_tensor
-from trojanzoo.utils.output import ansi, prints, output_iter
-from trojanzoo.utils import get_name, AverageMeter
+from trojanzoo.utils import get_name
+from trojanzoo.utils.output import ansi, get_ansi_len, prints, output_iter
+from trojanzoo.utils.logger import MetricLogger, SmoothedValue
 
 
 import torch
@@ -26,7 +27,6 @@ from collections import OrderedDict
 from collections.abc import Callable, Iterable    # TODO: callable (many places) (wait for python update)
 from typing import Generator, Iterator, Mapping, Set, Union, Optional
 
-InputType = Union[torch.Tensor, tuple]
 # redirect = Indent_Redirect(buffer=True, indent=0)
 
 
@@ -43,13 +43,13 @@ class _Model(nn.Module):
 
         self.features = self.define_features()   # feature extractor
         self.pool = nn.AdaptiveAvgPool2d((1, 1))  # average pooling
-        self.flatten = nn.Flatten(start_dim=1)
+        self.flatten = nn.Flatten()
         self.classifier = self.define_classifier()  # classifier
 
     # forward method
     # input: (batch_size, channels, height, width)
     # output: (batch_size, logits)
-    def forward(self, x: InputType) -> torch.Tensor:
+    def forward(self, x: torch.Tensor) -> torch.Tensor:
         # if x.shape is (channels, height, width)
         # (channels, height, width) ==> (batch_size: 1, channels, height, width)
         x = self.get_final_fm(x)
@@ -58,10 +58,10 @@ class _Model(nn.Module):
 
     # input: (batch_size, channels, height, width)
     # output: (batch_size, [feature_map])
-    def get_fm(self, x: InputType) -> torch.Tensor:
+    def get_fm(self, x: torch.Tensor) -> torch.Tensor:
         return self.features(x)
 
-    def get_final_fm(self, x: InputType) -> torch.Tensor:
+    def get_final_fm(self, x: torch.Tensor) -> torch.Tensor:
         x = self.get_fm(x)
         x = self.pool(x)
         x = self.flatten(x)
@@ -107,8 +107,6 @@ class Model:
     def add_argument(group: argparse._ArgumentGroup):
         group.add_argument('-m', '--model', dest='model_name',
                            help='model name, defaults to config[model][default_model]')
-        group.add_argument('--layer', dest='layer', type=int,
-                           help='layer (optional, maybe embedded in --model)')
         group.add_argument('--suffix', dest='suffix',
                            help='model name suffix, e.g. _adv_train')
         group.add_argument('--pretrain', dest='pretrain', action='store_true',
@@ -163,19 +161,19 @@ class Model:
         self.criterion = self.define_criterion(weight=to_tensor(loss_weights))
         self.softmax = nn.Softmax(dim=1)
         self._model = model_class(num_classes=num_classes, **kwargs)
+        self.model = self.get_parallel_model()
         self.activate_params([])
         if official:
             self.load('official')
         if pretrain:
             self.load()
-        self.model = self.get_parallel_model()
         self.eval()
-        if env['num_gpus']:  # TODO: might be useless if we set map_location correctly
+        if env['num_gpus']:
             self.cuda()
 
     # ----------------- Forward Operations ----------------------#
 
-    def get_logits(self, _input: InputType, randomized_smooth: bool = None,
+    def get_logits(self, _input: torch.Tensor, randomized_smooth: bool = None,
                    rs_sigma: float = None, rs_n: int = None, **kwargs) -> torch.Tensor:
         randomized_smooth = randomized_smooth if randomized_smooth is not None else self.randomized_smooth
         if randomized_smooth:
@@ -192,23 +190,24 @@ class Model:
         else:
             return self.model(_input, **kwargs)
 
-    def get_prob(self, _input: InputType, **kwargs) -> torch.Tensor:
-        return self.softmax(self.get_logits(_input, **kwargs))
+    def get_prob(self, _input: torch.Tensor, **kwargs) -> torch.Tensor:
+        return self.softmax(self(_input, **kwargs))
 
-    def get_final_fm(self, _input: InputType, **kwargs) -> torch.Tensor:
+    def get_final_fm(self, _input: torch.Tensor, **kwargs) -> torch.Tensor:
         return self._model.get_final_fm(_input, **kwargs)
 
-    def get_target_prob(self, _input: InputType, target: Union[torch.Tensor, list[int]],
+    def get_target_prob(self, _input: torch.Tensor, target: Union[torch.Tensor, list[int]],
                         **kwargs) -> torch.Tensor:
         if isinstance(target, list):
             target = torch.tensor(target, device=_input.device)
         return self.get_prob(_input, **kwargs).gather(dim=1, index=target.unsqueeze(1)).flatten()
 
-    def get_class(self, _input: InputType, **kwargs) -> torch.Tensor:
-        return self.get_logits(_input, **kwargs).argmax(dim=-1)
+    def get_class(self, _input: torch.Tensor, **kwargs) -> torch.Tensor:
+        return self(_input, **kwargs).argmax(dim=-1)
 
-    def loss(self, _input: InputType, _label: torch.Tensor, **kwargs) -> torch.Tensor:
-        _output = self(_input, **kwargs)
+    def loss(self, _input: torch.Tensor = None, _label: torch.Tensor = None, _output: torch.Tensor = None, **kwargs) -> torch.Tensor:
+        if _output is None:
+            _output = self(_input, **kwargs)
         return self.criterion(_output, _label)
 
     # -------------------------------------------------------- #
@@ -223,24 +222,24 @@ class Model:
 
     def define_optimizer(self, parameters: Union[str, Iterator[nn.Parameter]] = 'full',
                          OptimType: Union[str, type[Optimizer]] = None,
-                         lr_scheduler: bool = True,
-                         lr: float = 0.1, lr_decay_step: int = 30,
+                         lr: float = 0.1, momentum: float = 0.0, weight_decay: float = 0.0,
+                         lr_scheduler: bool = True, lr_step_size: int = 50, lr_gamma: float = 0.1,
                          **kwargs) -> tuple[Optimizer, _LRScheduler]:
+        kwargs['momentum'] = momentum
+        kwargs['weight_decay'] = weight_decay
         if isinstance(parameters, str):
             parameters = self.get_parameter_from_name(name=parameters)
         if not isinstance(parameters, Iterable):
             raise TypeError(f'{type(parameters)=}    {parameters=}')
         if isinstance(OptimType, str):
             OptimType: type[Optimizer] = getattr(torch.optim, OptimType)
-        if len(kwargs) == 0 and OptimType == torch.optim.SGD:
-            kwargs = {'momentum': 0.9, 'weight_decay': 2e-4, 'nesterov': True}
+        keys = OptimType.__init__.__code__.co_varnames
+        kwargs = {k: v for k, v in kwargs.items() if k in keys}
         optimizer = OptimType(parameters, lr, **kwargs)
         _lr_scheduler: _LRScheduler = None
         if lr_scheduler:
             _lr_scheduler = torch.optim.lr_scheduler.StepLR(
-                optimizer, step_size=lr_decay_step, gamma=0.1)
-            # optimizer = optim.lr_scheduler.MultiStepLR(
-            #     optimizer, milestones=[150, 250], gamma=0.1)
+                optimizer, step_size=lr_step_size, gamma=lr_gamma)
         return optimizer, _lr_scheduler
 
     # define loss function
@@ -256,65 +255,69 @@ class Model:
     # full: (default: False) whether save feature extractor.
     # output: (default: False) whether output help information.
     def load(self, file_path: str = None, folder_path: str = None, suffix: str = None,
-             map_location: Union[str, Callable, torch.device, dict] = 'default',
+             map_location: Union[str, Callable, torch.device, dict] = 'cpu',
              component: str = '', strict: bool = True,
              verbose: bool = False, indent: int = 0, **kwargs):
-        map_location = map_location if map_location != 'default' else env['device']
-        if file_path is None:
-            folder_path = folder_path if folder_path is not None else self.folder_path
-            suffix = suffix if suffix is not None else self.suffix
-            file_path = os.path.normpath(os.path.join(folder_path, f'{self.name}{suffix}.pth'))
-        if file_path == 'official':   # TODO
-            _dict = self.get_official_weights(map_location=map_location)
-            last_bias_value = next(reversed(_dict.values()))   # TODO: make sure
-            if self.num_classes != len(last_bias_value) and component != 'features':
-                strict = False
-                _dict.popitem()
-                _dict.popitem()
-        else:
-            try:
-                # TODO: type annotation might change? dict[str, torch.Tensor]
-                _dict: OrderedDict[str, torch.Tensor] = torch.load(file_path, map_location=map_location, **kwargs)
-            except Exception as e:
-                print(f'{file_path=}')
-                raise e
-        module = self._model
-        if component == 'features':
-            module = self._model.features
-            _dict = OrderedDict([(key.removeprefix('features.'), value) for key, value in _dict.items()])
-        elif component == 'classifier':
-            module = self._model.classifier
-            _dict = OrderedDict([(key.removeprefix('classifier.'), value) for key, value in _dict.items()])
-        else:
-            assert component == '', f'{component=}'
-        module.load_state_dict(_dict, strict=strict)
-        if verbose:
-            prints(f'Model {self.name} loaded from: {file_path}', indent=indent)
+        with torch.no_grad():
+            map_location = map_location if map_location != 'default' else env['device']
+            if file_path is None:
+                folder_path = folder_path if folder_path is not None else self.folder_path
+                suffix = suffix if suffix is not None else self.suffix
+                file_path = os.path.normpath(os.path.join(folder_path, f'{self.name}{suffix}.pth'))
+            if file_path == 'official':   # TODO
+                _dict = self.get_official_weights(map_location=map_location)
+                last_bias_value = next(reversed(_dict.values()))   # TODO: make sure
+                if self.num_classes != len(last_bias_value) and component != 'features':
+                    strict = False
+                    _dict.popitem()
+                    _dict.popitem()
+            else:
+                try:
+                    # TODO: type annotation might change? dict[str, torch.Tensor]
+                    _dict: OrderedDict[str, torch.Tensor] = torch.load(file_path, map_location=map_location, **kwargs)
+                except Exception as e:
+                    print(f'{file_path=}')
+                    raise e
+            module = self._model
+            if component == 'features':
+                module = self._model.features
+                _dict = OrderedDict([(key.removeprefix('features.'), value) for key, value in _dict.items()])
+            elif component == 'classifier':
+                module = self._model.classifier
+                _dict = OrderedDict([(key.removeprefix('classifier.'), value) for key, value in _dict.items()])
+            else:
+                assert component == '', f'{component=}'
+            module.load_state_dict(_dict, strict=strict)
+            if verbose:
+                prints(f'Model {self.name} loaded from: {file_path}', indent=indent)
+            if env['num_gpus']:
+                self.cuda()
 
     # file_path: (default: '') if '', use the default path.
     # full: (default: False) whether save feature extractor.
     def save(self, file_path: str = None, folder_path: str = None, suffix: str = None,
              component: str = '', verbose: bool = False, indent: int = 0, **kwargs):
-        if file_path is None:
-            folder_path = folder_path if folder_path is not None else self.folder_path
-            suffix = suffix if suffix is not None else self.suffix
-            file_path = os.path.normpath(os.path.join(folder_path, f'{self.name}{suffix}.pth'))
-        else:
-            folder_path = os.path.dirname(file_path)
-        if not os.path.exists(folder_path):
-            os.makedirs(folder_path)
-        # TODO: type annotation might change? dict[str, torch.Tensor]
-        module = self._model
-        if component == 'features':
-            module = self._model.features
-        elif component == 'classifier':
-            module = self._model.classifier
-        else:
-            assert component == '', f'{component=}'
-        _dict: OrderedDict[str, torch.Tensor] = module.state_dict(prefix=component)
-        torch.save(_dict, file_path, **kwargs)
-        if verbose:
-            prints(f'Model {self.name} saved at: {file_path}', indent=indent)
+        with torch.no_grad():
+            if file_path is None:
+                folder_path = folder_path if folder_path is not None else self.folder_path
+                suffix = suffix if suffix is not None else self.suffix
+                file_path = os.path.normpath(os.path.join(folder_path, f'{self.name}{suffix}.pth'))
+            else:
+                folder_path = os.path.dirname(file_path)
+            if not os.path.exists(folder_path):
+                os.makedirs(folder_path)
+            # TODO: type annotation might change? dict[str, torch.Tensor]
+            module = self._model
+            if component == 'features':
+                module = self._model.features
+            elif component == 'classifier':
+                module = self._model.classifier
+            else:
+                assert component == '', f'{component=}'
+            _dict: OrderedDict[str, torch.Tensor] = module.state_dict(prefix=component)
+            torch.save(_dict, file_path, **kwargs)
+            if verbose:
+                prints(f'Model {self.name} saved at: {file_path}', indent=indent)
 
     # define in concrete model class.
     # TODO: maybe write some generic style? model_url?
@@ -324,12 +327,13 @@ class Model:
 
     # -----------------------------------Train and Validate------------------------------------ #
     def _train(self, epoch: int, optimizer: Optimizer, lr_scheduler: _LRScheduler = None,
-               validate_interval: int = 10, save: bool = False, amp: bool = False, verbose: bool = True, indent: int = 0,
+               validate_interval: int = 10, save: bool = False, amp: bool = False,
                loader_train: torch.utils.data.DataLoader = None, loader_valid: torch.utils.data.DataLoader = None,
-               get_data_fn: Callable[..., tuple[InputType, torch.Tensor]] = None,
-               loss_fn: Callable[..., torch.Tensor] = None,
+               get_data_fn: Callable[..., tuple[torch.Tensor, torch.Tensor]] = None,
+               loss_fn: Callable[[torch.Tensor, torch.Tensor, torch.Tensor], torch.Tensor] = None,
                validate_func: Callable[..., tuple[float, ...]] = None, epoch_func: Callable[[], None] = None,
-               save_fn: Callable = None, file_path: str = None, folder_path: str = None, suffix: str = None, **kwargs):
+               save_fn: Callable = None, file_path: str = None, folder_path: str = None, suffix: str = None,
+               verbose: bool = True, indent: int = 0, **kwargs):
         loader_train = loader_train if loader_train is not None else self.dataset.loader['train']
         get_data_fn = get_data_fn if get_data_fn is not None else self.get_data
         loss_fn = loss_fn if loss_fn is not None else self.loss
@@ -337,75 +341,66 @@ class Model:
         save_fn = save_fn if save_fn is not None else self.save
 
         scaler: torch.cuda.amp.GradScaler = None
-        if amp and env['num_gpus']:
+        if not env['num_gpus']:
+            amp = False
+        if amp:
             scaler = torch.cuda.amp.GradScaler()
-        _, best_acc, _ = validate_func(loader=loader_valid, get_data_fn=get_data_fn, loss_fn=loss_fn,
-                                       verbose=verbose, indent=indent, **kwargs)
-        losses = AverageMeter('Loss')
-        top1 = AverageMeter('Acc@1')
-        top5 = AverageMeter('Acc@5')
+        _, best_acc = validate_func(loader=loader_valid, get_data_fn=get_data_fn, loss_fn=loss_fn,
+                                    verbose=verbose, indent=indent, **kwargs)
+
         params: list[list[nn.Parameter]] = [param_group['params'] for param_group in optimizer.param_groups]
         for _epoch in range(epoch):
             if epoch_func is not None:
                 self.activate_params([])
                 epoch_func()
                 self.activate_params(params)
-            losses.reset()
-            top1.reset()
-            top5.reset()
-            epoch_start = time.perf_counter()
-            loader = loader_train
-            if verbose and env['tqdm']:
-                loader = tqdm(loader_train)
+            logger = MetricLogger()
+            logger.meters['loss'] = SmoothedValue()
+            logger.meters['top1'] = SmoothedValue()
+            logger.meters['top5'] = SmoothedValue()
+            loader_epoch = loader_train
+            if verbose:
+                header = '{blue_light}Epoch: {0}{reset}'.format(
+                    output_iter(_epoch + 1, epoch), **ansi)
+                header = header.ljust(30 + get_ansi_len(header))
+                if env['tqdm']:
+                    header = '{upline}{clear_line}'.format(**ansi) + header
+                    loader_epoch = tqdm(loader_epoch)
+                loader_epoch = logger.log_every(loader_epoch, header=header, indent=indent)
             self.train()
             self.activate_params(params)
             optimizer.zero_grad()
-            for data in loader:
+            for data in loader_epoch:
                 # data_time.update(time.perf_counter() - end)
                 _input, _label = get_data_fn(data, mode='train')
-                if amp and env['num_gpus']:
-                    loss = loss_fn(_input, _label, amp=True)
+                _output = self(_input, amp=amp)
+                loss = loss_fn(_input, _label, _output=_output)
+                if amp:
                     scaler.scale(loss).backward()
                     scaler.step(optimizer)
                     scaler.update()
                 else:
-                    loss = loss_fn(_input, _label)
                     loss.backward()
                     optimizer.step()
                 optimizer.zero_grad()
-                with torch.no_grad():
-                    _output = self.get_logits(_input)
                 acc1, acc5 = self.accuracy(_output, _label, topk=(1, 5))
                 batch_size = int(_label.size(0))
-                losses.update(loss.item(), batch_size)
-                top1.update(acc1, batch_size)
-                top5.update(acc5, batch_size)
-                empty_cache()
-            epoch_time = str(datetime.timedelta(seconds=int(
-                time.perf_counter() - epoch_start)))
+                logger.meters['loss'].update(float(loss), batch_size)
+                logger.meters['top1'].update(acc1, batch_size)
+                logger.meters['top5'].update(acc5, batch_size)
+                empty_cache()   # TODO: should it be outside of the dataloader loop?
             self.eval()
             self.activate_params([])
-            if verbose:
-                pre_str = '{blue_light}Epoch: {0}{reset}'.format(
-                    output_iter(_epoch + 1, epoch), **ansi).ljust(64 if env['color'] else 35)
-                _str = ' '.join([
-                    f'Loss: {losses.avg:.4f},'.ljust(20),
-                    f'Top1 Acc: {top1.avg:.3f}, '.ljust(20),
-                    f'Top5 Acc: {top5.avg:.3f},'.ljust(20),
-                    f'Time: {epoch_time},'.ljust(20),
-                ])
-                prints(pre_str, _str, prefix='{upline}{clear_line}'.format(**ansi) if env['tqdm'] else '',
-                       indent=indent)
             if lr_scheduler:
                 lr_scheduler.step()
-
             if validate_interval != 0:
                 if (_epoch + 1) % validate_interval == 0 or _epoch == epoch - 1:
-                    _, cur_acc, _ = validate_func(loader=loader_valid, get_data_fn=get_data_fn, loss_fn=loss_fn,
-                                                  verbose=verbose, indent=indent, **kwargs)
+                    _, cur_acc = validate_func(loader=loader_valid, get_data_fn=get_data_fn, loss_fn=loss_fn,
+                                               verbose=verbose, indent=indent, **kwargs)
                     if cur_acc >= best_acc:
-                        prints('best result update!', indent=indent)
-                        prints(f'Current Acc: {cur_acc:.3f}    Previous Best Acc: {best_acc:.3f}', indent=indent)
+                        if verbose:
+                            prints('{green}best result update!{reset}'.format(**ansi), indent=indent)
+                            prints(f'Current Acc: {cur_acc:.3f}    Previous Best Acc: {best_acc:.3f}', indent=indent)
                         best_acc = cur_acc
                         if save:
                             save_fn(file_path=file_path, folder_path=folder_path, suffix=suffix, verbose=verbose)
@@ -415,45 +410,41 @@ class Model:
 
     def _validate(self, full=True, print_prefix='Validate', indent=0, verbose=True,
                   loader: torch.utils.data.DataLoader = None,
-                  get_data_fn: Callable = None, loss_fn: Callable[..., float] = None, **kwargs) -> tuple[float, ...]:
+                  get_data_fn: Callable[..., tuple[torch.Tensor, torch.Tensor]] = None,
+                  loss_fn: Callable[[torch.Tensor, torch.Tensor, torch.Tensor], torch.Tensor] = None,
+                  **kwargs) -> tuple[float, float]:
         self.eval()
         if loader is None:
             loader = self.dataset.loader['valid'] if full else self.dataset.loader['valid2']
         get_data_fn = get_data_fn if get_data_fn is not None else self.get_data
         loss_fn = loss_fn if loss_fn is not None else self.loss
-        losses = AverageMeter('Loss', ':.4e')
-        top1 = AverageMeter('Acc@1', ':6.2f')
-        top5 = AverageMeter('Acc@5', ':6.2f')
-        epoch_start = time.perf_counter()
-        if verbose and env['tqdm']:
-            loader = tqdm(loader)
-        for data in loader:
-            _input, _label = get_data_fn(data, mode='valid', **kwargs)
-            with torch.no_grad():
-                loss = loss_fn(_input, _label)
-                _output = self.get_logits(_input)
-            # measure accuracy and record loss
-            batch_size = int(_label.size(0))
-            losses.update(loss.item(), _label.size(0))
-            acc1, acc5 = self.accuracy(_output, _label, topk=(1, 5))
-            top1.update(acc1, batch_size)
-            top5.update(acc5, batch_size)
-        epoch_time = str(datetime.timedelta(seconds=int(
-            time.perf_counter() - epoch_start)))
+        logger = MetricLogger()
+        logger.meters['loss'] = SmoothedValue()
+        logger.meters['top1'] = SmoothedValue()
+        logger.meters['top5'] = SmoothedValue()
+        loader_epoch = loader
         if verbose:
-            pre_str = '{yellow}{0}:{reset}'.format(print_prefix, **ansi).ljust(35)
-            _str = ' '.join([
-                f'Loss: {losses.avg:.4f},'.ljust(20),
-                f'Top1 Acc: {top1.avg:.3f}, '.ljust(20),
-                f'Top5 Acc: {top5.avg:.3f},'.ljust(20),
-                f'Time: {epoch_time},'.ljust(20),
-            ])
-            prints(pre_str, _str, prefix='{upline}{clear_line}'.format(**ansi) if env['tqdm'] else '', indent=indent)
-        return losses.avg, top1.avg, top5.avg
+            header = '{yellow}{0}{reset}'.format(print_prefix, **ansi)
+            header = header.ljust(max(len(print_prefix), 30) + get_ansi_len(header))
+            if env['tqdm']:
+                header = '{upline}{clear_line}'.format(**ansi) + header
+                loader_epoch = tqdm(loader_epoch)
+            loader_epoch = logger.log_every(loader_epoch, header=header, indent=indent)
+        with torch.no_grad():
+            for data in loader_epoch:
+                _input, _label = get_data_fn(data, mode='valid', **kwargs)
+                _output = self(_input)
+                loss = float(loss_fn(_input, _label, _output=_output))
+                acc1, acc5 = self.accuracy(_output, _label, topk=(1, 5))
+                batch_size = int(_label.size(0))
+                logger.meters['loss'].update(loss, batch_size)
+                logger.meters['top1'].update(acc1, batch_size)
+                logger.meters['top5'].update(acc5, batch_size)
+        return logger.meters['loss'].global_avg, logger.meters['top1'].global_avg
 
     # -------------------------------------------Utility--------------------------------------- #
 
-    def get_data(self, data: tuple[InputType, torch.Tensor], **kwargs):
+    def get_data(self, data: tuple[torch.Tensor, torch.Tensor], **kwargs):
         if self.dataset is not None:
             return self.dataset.get_data(data, **kwargs)
         else:
@@ -461,20 +452,20 @@ class Model:
 
     def accuracy(self, _output: torch.Tensor, _label: torch.Tensor,
                  topk: tuple[int] = (1, 5)) -> tuple[float, ...]:
-        """Computes the precision@k for the specified values of k"""
+        """Computes the accuracy over the k top predictions for the specified values of k"""
         with torch.no_grad():
             maxk = min(max(topk), self.num_classes)
-            batch_size = _label.shape[0]
+            batch_size = _label.size(0)
             _, pred = _output.topk(maxk, 1, True, True)
             pred = pred.t()
-            correct = pred.eq(_label.view(1, -1).expand_as(pred))
+            correct = pred.eq(_label[None])
             res: tuple[float, ...] = []
             for k in topk:
                 if k > self.num_classes:
                     res.append(100.0)
                 else:
-                    correct_k = correct[:k].flatten().float().sum(0, keepdim=True)
-                    res.append(float(correct_k.mul_(100.0 / batch_size)))
+                    correct_k = correct[:k].flatten().sum(dtype=torch.float32)
+                    res.append(correct_k * (100.0 / batch_size))
             return res
 
     def get_parameter_from_name(self, name: str = '') -> Iterator[nn.Parameter]:
@@ -495,15 +486,16 @@ class Model:
                 param.requires_grad_()
 
     # Need to overload for other packages (GNN) since they are calling their own nn.DataParallel.
+    # TODO: nn.parallel.DistributedDataParallel
     def get_parallel_model(self) -> Union[_Model, nn.DataParallel]:
         if env['num_gpus'] > 1:
             return nn.DataParallel(self._model)
         return self._model
 
-    @staticmethod
+    @ staticmethod
     def output_layer_information(layer: nn.Module, depth: int = 0, verbose: bool = True,
                                  indent: int = 0, tree_length: int = None):
-        tree_length = tree_length if tree_length is not None else 10 * (depth + 1)
+        tree_length = tree_length if tree_length is not None else 12 * (depth + 1)
         if depth > 0:
             for name, module in layer.named_children():
                 _str = '{blue_light}{0}{reset}'.format(name, **ansi)
@@ -516,9 +508,11 @@ class Model:
                     _str += item
                 prints(_str, indent=indent)
                 Model.output_layer_information(
-                    module, depth=depth - 1, indent=indent + 10, verbose=verbose, tree_length=tree_length)
+                    module, depth=depth - 1, indent=indent + 12, verbose=verbose, tree_length=tree_length)
 
-    def summary(self, depth: int = 2, verbose: bool = True, indent: int = 0, **kwargs):
+    def summary(self, depth: int = None, verbose: bool = True, indent: int = 0, **kwargs):
+        if depth is None:
+            depth = env['verbose']
         prints('{blue_light}{0:<20s}{reset} Parameters: '.format(self.name, **ansi), indent=indent)
         for key, value in self.param_list.items():
             prints('{green}{0:<20s}{reset}'.format(key, **ansi), indent=indent + 10)
@@ -529,7 +523,7 @@ class Model:
 
     # -----------------------------------------Reload------------------------------------------ #
 
-    def __call__(self, _input: InputType, amp: bool = False, **kwargs) -> torch.Tensor:
+    def __call__(self, _input: torch.Tensor, amp: bool = False, **kwargs) -> torch.Tensor:
         if amp:
             with torch.cuda.amp.autocast():
                 return self.get_logits(_input, **kwargs)
@@ -598,16 +592,16 @@ class Model:
 
     # ----------------------------------------------------------------------------------------- #
 
-    def remove_misclassify(self, data: tuple[InputType, torch.Tensor], **kwargs):
+    def remove_misclassify(self, data: tuple[torch.Tensor, torch.Tensor], **kwargs):
         with torch.no_grad():
             _input, _label = self.get_data(data, **kwargs)
             _classification = self.get_class(_input)
             repeat_idx = _classification.eq(_label)
         return _input[repeat_idx], _label[repeat_idx]
 
-    def generate_target(self, _input: InputType, idx: int = 1, same: bool = False) -> torch.Tensor:
+    def generate_target(self, _input: torch.Tensor, idx: int = 1, same: bool = False) -> torch.Tensor:
         with torch.no_grad():
-            _output = self.get_logits(_input)
+            _output = self(_input)
         target = _output.argsort(dim=-1, descending=True)[:, idx]
         if same:
             target = repeat_to_batch(target.mode(dim=0)[0], len(_input))
